# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.

__author__ = "Thinh Vu @thinh-vu in GitHub"
__version__ = "0.0.7"

from ur_gadget import *
from .database_connect import *
from .messaging_api import *
from .google_service import *
from .visualization import *
from .crm_system import *


import pandas as pd
import requests
import json
import io
import time
from datetime import datetime
# from .util import yaml_cred

def fb_ad_report(token, ad_account_id, report_level, date_preset, fields= 'date_start, date_stop, account_name, campaign_name, spend', start_date='', end_date='', filtering='', time_increment=1, api_version='v15.0', wait_time=10):
    """
        Return Facebook Ads report as a Dataframe using Facebook Marketing API.
    Args:
        token (:obj:`str`, required): The Marketing API token string. This value can be read from a yaml config file.
        ad_account_id (:obj:`str`, required): Ad Account ID. Ex: '1210183442581521'
        report_level (:obj:`str`, required): 'account' or 'campaign', 'adset', or 'ad'
        fields (:obj:`str`, optional): Default value = 'date_start, date_stop, account_name, campaign_name, spend'. See the full available list of fields [here](https://developers.facebook.com/docs/marketing-api/insights/parameters/v15.0)
        date_preset (:obj:`str`, optional): This setting will be ignored when date range (`start_date` and `end_date` were input) is specified. Use one of these values: 'today', 'yesterday', 'this_month', 'last_month', 'this_quarter', 'maximum', 'data_maximum', 'last_3d', 'last_7d', 'last_14d', 'last_28d', 'last_30d', 'last_90d', etc. See the full list [here](https://developers.facebook.com/docs/marketing-api/insights/parameters#param)
        start_date (:obj:`str`, optional): Input date value in the format of YYYY-MM-D. Ex: '2022-11-13'
        end_date (:obj:`str`, optional): Input date value in the format of YYYY-MM-D. Ex: '2022-11-13'
        filtering (:obj:`str`, optional): No filtering applied as default. Read the full reference [here](https://developers.facebook.com/docs/marketing-api/insights/parameters#param)
        time_increment (:obj:`str`, optional): Default value = 1 to get the daily report data. 
        time_increment (:obj:`str`, optional): Default value = 'v15.0' as the current value. You can find the latest version by visiting the reference site [here](https://developers.facebook.com/docs/marketing-api/insights/parameters#param)
        wait_time (:obj:`int`, optional): Wait time to wait for the report to be generated as a CSV file before loading to the Dataframe. Increase wait time in seconds when you download a report with a long date range
    """
    date_range = "{'since':'" + start_date + "','until':'" + end_date + "'}"
    async_url = f"https://graph.facebook.com/{api_version}/act_{ad_account_id}/insights?level={report_level}&fields={fields}&date_preset={date_preset}&time_range={date_range}&filtering={filtering}&time_increment={time_increment}&access_token={token}"
    report_run_id = json.loads(json.dumps(requests.post(async_url).json())).get('report_run_id') # extract report_run_id
    time.sleep(wait_time) # add sleep time to wait for data download
    export_url = f"https://www.facebook.com/ads/ads_insights/export_report?report_run_id={report_run_id}&format=csv&locale=en_US&access_token={token}"
    rawData = requests.get(export_url).content
    df = pd.read_csv(io.StringIO(rawData.decode('utf-8')))
    return df



# Copyright 2022 Thinh Vu @ GitHub
# See LICENSE for details.
import pandas as pd
from oauth2client.service_account import ServiceAccountCredentials
import gspread
from df2gspread import df2gspread as d2g

from google.oauth2 import service_account # to use gg search console
from googleapiclient.discovery import build
import json
import requests

# Authentication
def google_auth(json_cred_path):
    """
    Return a creds and gc to use Authenticate the Google service
    Args:
        json_cred_path (:obj:`str`, required): The path to the service account credential in json format.
    """
    scope = ['https://spreadsheets.google.com/feeds','https://www.googleapis.com/auth/drive']
    creds = ServiceAccountCredentials.from_json_keyfile_name(json_cred_path, scope)
    gc = gspread.authorize(creds)
    return creds, gc

# Google Sheets
def read_gsheet(sheet_id, sheet_name, gc):
    """
    Read a Google Sheets file to return a DataFrame. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        gc (:obj, required): gc be assigned as to the result of google_auth() function.
    """
    wks = gc.open_by_key(sheet_id)
    worksheet = wks.worksheet(sheet_name)
    df = pd.DataFrame(worksheet.get_all_records())
    return df

def save_gsheet (dataframe, sheet_id, sheet_name, creds, row_names=False, clean=False, df_size=False, start_cell='A1'):
    """
    Export a DataFrame to Google Sheets file. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj, required): Target DataFrame to export.
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        creds (:obj, required): creds be assigned as to the result of google_auth() function.
        rows_name (:obj:`boolean`, required): False as the default value.
        clean (:obj, required): False as the default value.
        df_size(:obj, required): False as the default value.
        start_cell (:obj, required): 'A1' as the default value.
    """
    d2g.upload(dataframe, sheet_id, sheet_name, credentials=creds, row_names=row_names, clean = clean, df_size = df_size, start_cell=start_cell)
    'https://docs.google.com/spreadsheets/d/' + sheet_id

def save_gspread (dataframe, sheet_id, sheet_name, gc):
    """
    Read a Google Sheets file to return a DataFrame. Given the fact that the target Google Sheets file has been shared to your Google Service Account.
    Args:
        sheet_id (:obj:`str`, required): Google Sheets ID. Ex: `YOUR_SHEET_ID` in the URI`https://docs.google.com/spreadsheets/d/YOUR_SHEET_ID/edit`
        sheet_name (:obj:`str`, required): Name of the worksheet
        gc (:obj, required): gc be assigned as to the result of google_auth() function.
    """
    try:
        wks = gc.open_by_key(sheet_id)
        wks.worksheet(sheet_name).update([dataframe.columns.values.tolist()] + dataframe.values.tolist())
        message = 'Data Frame has been exported to https://docs.google.com/spreadsheets/d/' + sheet_id
    except:
        wks = gc.open_by_key(sheet_id)
        rows, columns = dataframe.shape
        wks.add_worksheet(title=sheet_name, rows=rows, cols=columns)
        wks.worksheet(sheet_name).update([dataframe.columns.values.tolist()] + dataframe.values.tolist())
        message = 'Data Frame has been exported to https://docs.google.com/spreadsheets/d/' + sheet_id
    return message

# Google Search Console
def connect(key):
    """Create a connection to the Google Search Console API and return service object.
    Args:
        key (string): Google Search Console JSON client secrets path.
    """
    scope = ['https://www.googleapis.com/auth/webmasters']
    credentials = service_account.Credentials.from_service_account_file(key, scopes=scope)
    service = build(
        'searchconsole',
        'v1',
        credentials=credentials
    )
    return service

def query(service, site_url, payload):
    """Run a query on the Google Search Console API and return a dataframe of results.
    Args:
        service (object): Service object from connect()
        site_url (string): URL of Google Search Console property
        payload (dict): API query payload dictionary    
    """
    response = service.searchanalytics().query(siteUrl=site_url, body=payload).execute()
    results = []
    for row in response['rows']:    
        data = {}
        for i in range(len(payload['dimensions'])):
            data[payload['dimensions'][i]] = row['keys'][i]
        data['clicks'] = row['clicks']
        data['impressions'] = row['impressions']
        data['ctr'] = round(row['ctr'] * 100, 2)
        data['position'] = round(row['position'], 2)        
        results.append(data)
    return pd.DataFrame.from_dict(results)

def search_analysis_extract(key, site_url, start_date, end_date, dimension=["date","page","device","query"], limit=1000, start=0):
    """Extract Google Search Console data to a dataframe using connect and query functions defined above.
    Args:
        key (:obj:`str`, required): Google Search Console JSON client secrets path.
        site_url (:obj:`str`, required): URL of Google Search Console property
        start_date (:obj:`str`, required): Start date to extract data
        end_date (:obj:`str`, required): Start date to extract data
        dimension (:obj:`list`, optional): Available dimension: date, page, device, query, search-appearance (this one can only sit standalone). Default value =  ["date","page","device","query"]
        limit (:obj:`int`, optional): Maximum limit is 25000. Default value = 1000.
        start (:obj:`int`, optional): Start from the first result, default value = 0. 
    """
    service = connect(key)
    payload = {
        'startDate': f"{start_date}",
        'endDate': f"{end_date}",
        'dimensions': dimension, # 'search-appearance' can only sit standalone
        'rowLimit': f"{limit}",
        'startRow': f"{start}"
    }
    site_url = f"{site_url}"
    df = query(service, site_url, payload)
    return df